%% Author Information
% Written by Tanmay Majmudar, BS
%   MD Candidate, Class of 2024, Drexel University College of Medicine
%   Research Assistant, Orthopedic Biomechanics Laboratory, Drexel University School of Biomedical Engineering, Science, and Health Systems
%   Research Assistant, Ortho and Neuro Biomechanics Laboratory, Department of Biomedical Enginering, Widener University School of Engineering
% Email: tvm34@drexel.edu

%% Purpose
% This MATLAB function will calculate a series of biomechanical paramters to characterize the failure responses of various biological tissue,
% including trachea, peripheral nerve, bone etc. 

% Accompanies main script: "main_relax_failure.m". Please refer to comments in the main script for further details. 

%% Main Function
function failureParams = calc_relax_failure(data,header,A0,L0,postRelaxFlag)
% Initialize variables
if postRelaxFlag
    disp = data(:,contains(header,'Position','IgnoreCase',true))-data(1,contains(header,'Position','IgnoreCase',true));
    L0 = L0+data(1,contains(header,'Position','IgnoreCase',true));
else
    disp = data(:,contains(header,'Position','IgnoreCase',true));
end
strain = disp/L0;
load = data(:,contains(header,'Load','IgnoreCase',true));
time = data(:,contains(header,'Time','IgnoreCase',true));
stress = load/A0;

% Initialize output array
failureParams = nan(1,8);

% Remove all data after maximum load/stress
trncLoad = load; trncDisp = disp;
[failureParams(3),maxIdx] = max(load); % Protocol step 4.3.2
trncLoad(maxIdx:end) = []; trncDisp(maxIdx:end) = []; % Protocol step 4.3.3

% Calculate Stiffness; Also start Protocol step 4.3.4
f = figure;
p = plot(trncDisp,trncLoad);
hold on
title({'LEFT CLICK to select two points to approximate the linear region of the curve.';...
    'Then press ENTER to continue.';...
    'If a selected point is not desirable or more than two points are selected, press BACKSPACE or DELETE to remove.'})
xlabel('Displacement (mm)');
ylabel('Load (N)');
set(gcf,'WindowState','Maximized')
[xs,ys] = getpts();
while numel(xs)~=2
    title({'Please try again and ensure instructions are executed properly.';'LEFT CLICK to select two points to approximate the linear region of the curve.';...
    'Then press ENTER to continue.';...
    'If a selected point is not desirable or more than two points are selected, press BACKSPACE or DELETE to remove.'});
    [xs,ys] = getpts();
end
close(f); clear f
failureParams(1) = diff(ys)/diff(xs);

% Calculate Young's Modulus based on stiffness
failureParams(2) = (diff(ys)/A0)/(diff(xs)/L0); % End Protocol step 4.3.4

% Calculate displacement @ failure load, failure stress, strain @ failure stress, time @ failure load/stress
failureParams(4) = disp(maxIdx); % Protocol step 4.3.2
failureParams(5) = stress(maxIdx); % Protocol step 4.3.2
failureParams(6) = strain(maxIdx); % Protocol step 4.3.2
failureParams(7) = time(maxIdx);

% Calculate Modulus of Toughness
failureParams(8) = trapz(strain(stress>0),stress(stress>0));
end