%% Author Information
% Written by Tanmay Majmudar, BS
%   MD Candidate, Class of 2024, Drexel University College of Medicine
%   Research Assistant, Orthopedic Biomechanics Laboratory, Drexel University School of Biomedical Engineering, Science, and Health Systems
%   Research Assistant, Ortho and Neuro Biomechanics Laboratory, Department of Biomedical Enginering, Widener University School of Engineering
% Email: tvm34@drexel.edu

%% Purpose
% This MATLAB script will calculate a series of basic biomechanical parameters to characterize the stress relaxation and 
% failure responses of tracheal tissue. However, this code can be easily modified to compute similar metrics for other biological tissue, 
% including peripheral nerve, bone, etc. Lines where modifications need to be made will be clearly marked in the script. 

% A tutorial video has also been created that explains this code and should be viewed prior to running. 

% Sample specific parameters: 
% (1) Average cross-sectional area (CSA) in mm^2
% (2) Initial Length (mm)

% Stress relaxation response parameters: 
% (1) Peak Load (N)
% (2) Load at End of Relaxation Testing (N)
% (3) % Peak Load at End of Relaxation Testing (%)
% (4) % Decrease in Load at End of Relaxation Testing (%)
% (5) Peak Stress (MPa)
% (6) Stress at End of Relaxation Testing (N)
% (7) % Peak Stress at End of Relaxation Testing (%)
% (8) % Decrease in Stress at End of Relaxation Testing (%)
% (9) G0 (MPa)
% (10) G1 (MPa)
% (11) Tau 1 (sec)
% (12) G2 (MPa)
% (13) Tau 2 (sec)
% (14) Adjusted R^2

% Parameters #10-13 listed above are derived from curve fitting the experimental stress relaxation data to a two-term Prony series
% exponential decay model using non-linear least squares regression. Please refer to following for additional details: 
% Eskandari, M., Arvayo, A. L. & Levenston, M. E. Mechanical properties of the airway tree: heterogeneous and anisotropic 
%   pseudoelastic and viscoelastic tissue responses. J Appl Physiol (1985). 125 (3), 878-888, (2018).

% Failure response parameters
% (1) Stiffness (N/mm)
% (2) Young's Modulus (MPa)
% (3) Failure Load (N)
% (4) Displacement at Failure Load (mm)
% (5) Failure Stress (MPa)
% (6) Strain at Failure Stress (mm/mm)
% (7) Time at Failure Load/Stress (sec)
% (8) Modulus of Toughness (MPa)

% Auxiliary function required: 
% (1) calc_relax_failure.m

% Output format
% Excel workbook named in the following format: relax_failure_results_mmddyy where "mmddyy" represents date when this program was run

%% Initialize
clear all; close all; clc;

% Prompt user to input pre-stretch strain levels to be analyzed (i.e., insert the strain levels that samples will be held at for relaxation testing)
prompt = 'Enter comma-separated fixed elongation levels (in %) for various stress relaxation experimental groups: ';
dlgtitle = 'Input';
dims = [1 40];
definput = {'10'};
prestretch_levels = inputdlg(prompt,dlgtitle,dims,definput);
prestretch_levels = str2num(char(regexp(prestretch_levels{1},'\d*','match')));

% Prompt user to input stress relaxation testing durations to be analyzed
prompt = 'Enter comma-separated stress relaxation testing durations (in seconds) for various stress relaxation experimental groups: ';
dlgtitle = 'Input';
dims = [1 40];
definput = {'300'};
relax_durations = inputdlg(prompt,dlgtitle,dims,definput);
relax_durations = str2num(char(regexp(relax_durations{1},'\d*','match')));

%% Main script

% Sample specific parameters
outputParams_sample = {'Testing Date', 'Sample ID', 'Cross-Sectional Area', 'Initial Length (mm)','Failure Only? (Y/N)','Expected Pre-Stretch Strain Level',...
    'Observed Pre-Stretch Strain Level', 'Stress Relaxation Testing Time (sec)'};

% Stress relaxation response parameters
outputParams_relax = {'Peak Load (N)','Load at End of Relaxation Testing (N)','% Peak Load at End of Relaxation Testing (%)',...
    '% Decrease in Load at End of Relaxation Testing (%)','Peak Stress (MPa)','Stress at End of Relaxation Testing (N)',...
    '% Peak Stress at End of Relaxation Testing (%)', '% Decrease in Stress at End of Relaxation Testing (%)',...
    'G0 (MPa)','G1 (MPa)','Tau 1 (sec)','G2 (MPa)', 'Tau 2 (sec)','Adjusted R2'};

% Failure response parameters
outputParams_failure = {'Stiffness (N/mm)','Youngs Modulus (MPa)','Failure Load (N)','Displacement at Failure Load (mm)',...
    'Failure Stress (MPa)','Strain at Failure Stress (mm/mm)','Time at Failure Load/Stress (sec)','Modulus of Toughness (MPa)'};
    
% Create output excel workbook header
outDt = [outputParams_sample, outputParams_relax, outputParams_failure];

% Identify paths to Failure Only, Failure post-Relaxation, Relaxation folders
relaxFilesPath = fullfile(cd,'Relaxation');
failureRelaxFilesPath = fullfile(cd,'Failure (Post Relaxation)');
failureOnlyFilesPath = fullfile(cd,'Failure Only');

% Load excel file containing testing date information
% Please edit this excel workbook in the format of the provided example to include additional testing dates, if needed
testDateInfo = table2cell(readtable('testingDates.xlsx'));

% Remove backslashes from dates
for i = 1:size(testDateInfo,1)
    testDateInfo{i,1} = datestr(testDateInfo{i,1},'mmddyy');
end

% Loop through each testing date
tic
for d = 1:size(testDateInfo,1)
    currDate = testDateInfo{d,1};
    
    % Loop through each sample within the current testing date that underwent relaxation testing and then failure testing
    if testDateInfo{d,2} == 'Y'
        % Loop through each sample
        [~,sampleNames] = xlsfinfo(fullfile(relaxFilesPath,cat(2,currDate,'.xlsx')));
        for s = 1:length(sampleNames)
            [relaxDt,headers] = xlsread(fullfile(relaxFilesPath,cat(2,currDate,'.xlsx')),sampleNames{s});
            
            % Store general sample information
            currCell = cell(0,0);
            currCell{1,1} = currDate;
            currCell{1,2} = sampleNames{s};
            currCell{1,3} = relaxDt(1,contains(headers,'area','IgnoreCase',true));
            currCell{1,4} = relaxDt(1,contains(headers,'initial length','IgnoreCase',true));
            currCell{1,5} = 'N';
            
            % Within each testing date excel workbook, each tab should represent a given sample and needs to be named in the following format: 
            % [sample type]_[sample number]_[insert pre-stretch strain level]%
            % For example, in the current study, trachel specimen were pre-stretched axially or circumferentially to 10% pre-stretch
            % Therefore, tabs are named in the following format: "TA_1_10%" or "TC_1_10%" representing axial and circumferential loading, respectively
            % Another example may include different types of peripheral nerves pre-stretched to 10% or 20% strain. In this case, the tabs would named in
            % the following fashion: Tibial_1_10%, Tibial_2_20%, Sciatic_1_10%, Sciatic_2_20%
            % Please note no pre-stretch i.e., failure only group must be noted as 0% pre-stretch
            currCell{1,6} = str2double(regexp(sampleNames{s},'(?<=_)\d+(?=%$)','once','match'))/100;
            
            currCell{1,8} = relaxDt(end,contains(headers,'time','IgnoreCase',true));
            currCell{1,9} = relaxDt(1,contains(headers,'load','IgnoreCase',true)); % Protocol step 4.2.2
            currCell{1,10} = relaxDt(end,contains(headers,'load','IgnoreCase',true));
            currCell{1,11} = (relaxDt(end,contains(headers,'load','IgnoreCase',true))/relaxDt(1,contains(headers,'load','IgnoreCase',true)))*100;
            currCell{1,12} = 100-currCell{1,11}; % Protocol step 4.2.3
            
            strain = relaxDt(:,contains(headers,'Position','IgnoreCase',true))/currCell{1,4}; % Protocol step 4.3.1    
            stress = relaxDt(:,contains(headers,'load','IgnoreCase',true))/currCell{1,3}; % Protocol steps 4.2.1 and 4.3.1
            
            currCell{1,13} = stress(1); % Protocol step 4.2.2
            currCell{1,14} = stress(end);
            currCell{1,15} = (stress(end)/stress(1))*100;
            currCell{1,16} = 100-currCell{1,15}; % Protocol step 4.2.3
            
            % Lines 152-161 refer to Protocol step 4.2.4
            % Model tracheal stress relaxation response using a two-term prony series exponential decay model
            % Please refer to following for additional details: 
                % Eskandari, M., Arvayo, A. L. & Levenston, M. E. Mechanical properties of the airway tree: heterogeneous and anisotropic 
                % pseudoelastic and viscoelastic tissue responses. J Appl Physiol (1985). 125 (3), 878-888, (2018)
            % MODIFY INITIAL CONDITIONS AND LIMITS BASED ON TISSUE TYPE
            if contains(sampleNames{s},'TA') % Tracheal specimen under axial loading
                coeffs = lsqcurvefit(@subfun,[0.75,1,14.6,1,154.1],relaxDt(:,contains(headers,'Time','IgnoreCase',true)), stress/max(stress),[0,0,0,0,0],[1,1,100,1,1000]);
            else % Tracheal specimen under circumferential loading
                coeffs = lsqcurvefit(@subfun,[0.5,1,9.5,1,99.1],relaxDt(:,contains(headers,'Time','IgnoreCase',true)),stress/max(stress),[0,0,0,0,0],[1,1,100,1,1000]);
            end
            
            % Calculate Adjusted R^2 for model fit
            SSres = sum((subfun(coeffs,relaxDt(:,contains(headers,'Time','IgnoreCase',true)))-stress/max(stress)).^2);
            SStot = sum(((stress/max(stress))-mean(stress/max(stress))).^2);
            currCell{1,22} = (1-((1-(1 - SSres/SStot))*(numel(relaxDt(:,contains(headers,'Time','IgnoreCase',true)))-1))/(numel(relaxDt(:,contains(headers,'Time','IgnoreCase',true)))-numel(coeffs)-1))*100;
        
            % Store G0, G1, Tau 1, G2, Tau 2 in the output cell array
            for i = 17:21
                currCell{1,i} = coeffs(i-16);
            end

            % Retrieve the failure data from the current sample + calculate failure parameters + store
            try
                [failDt,headers_failure] = xlsread(fullfile(failureRelaxFilesPath,cat(2,currDate,'.xlsx')),sampleNames{s});
                currCell{1,7} = (failDt(1,contains(headers,'Position','IgnoreCase',true))/failDt(1,contains(headers,'Length','IgnoreCase',true)));
                currCell(1,23:30) = num2cell(calc_relax_failure(failDt,headers_failure,failDt(1,contains(headers_failure,'area','IgnoreCase',true)),failDt(1,contains(headers_failure,'length','IgnoreCase',true)),1));
            catch
                currCell(1,23:30) = {NaN};
                currCell{1,7} = currCell{1,6};
            end 
            
            % Concatenate currCell with outDt
            outDt = [outDt;currCell];
        end
    end
    
    % Loop through each sample within the current testing date that underwent failure testing only
    if testDateInfo{d,3} == 'Y'
        % Loop through each sample
        [~,sampleNames] = xlsfinfo(fullfile(failureOnlyFilesPath,cat(2,currDate,'.xlsx')));
        for s = 1:length(sampleNames)
            [failDt,headers_failure] = xlsread(fullfile(failureOnlyFilesPath,cat(2,currDate,'.xlsx')),sampleNames{s});
            
            % Store general sample information + Calculate failure params + store
            currCell = cell(0,0);
            currCell{1,1} = currDate;
            currCell{1,2} = sampleNames{s};
            currCell{1,3} = failDt(1,contains(headers_failure,'area','IgnoreCase',true));
            currCell{1,4} = failDt(1,contains(headers_failure,'length','IgnoreCase',true));
            currCell{1,5} = 'Y';
            currCell(1,6:22) = {NaN};
            currCell(1,23:30) = num2cell(calc_relax_failure(failDt,headers_failure,failDt(1,contains(headers_failure,'area','IgnoreCase',true)),failDt(1,contains(headers_failure,'length','IgnoreCase',true)),0));

            % Concatenate currCell with outDt
            outDt = [outDt;currCell];
        end
    end
toc
end

% Output data to an excel file
writetable(cell2table(outDt(2:end,:),'VariableNames',outDt(1,:)),sprintf('relax_failure_results_%s.xlsx',datestr(date,'mmddyy')))

% Print code completion statement
fprintf('Code successfully ran. Please see output Excel file in the MATLAB Current Folder\n')

%% Two Term Prony Series Relaxation Function
function yfit = subfun(params,t)
yfit = params(1)+params(2)*exp(-t/params(3))+params(4)*exp(-t/params(5));
end